" Generate usr_toc.txt from usr_*.txt help files
" Author: David Blanchet <davidDOTblanchetATfreeDOTfr>
"
" Usage:
"  1. Drop this script in the directory that contains help files
" then
"  2. Open this file with gvim then ":so %"
" or
"  2. "gvim -s usr_toc.vim"
"
" About:
" This script uses template file called "usr_toc.skeleton.txt", that
" contains the "usr_toc.txt" skeleton and special tags. These tags
" are used by this script to insert the suitable informations.
"
" Special tags should be structure as follows :
"   |
"   |XXX <type> <first>[..<last>]
"   |
"    ^
"    first column
"
" <type> is either "title" (only manual file title) or "full" (file 
" title + section titles).
"
" <first> is the only or first file of the list.
"
" <last> is the last file of the list. It separated from <first>
" argument with two dots "..". <last> must be greater than <first>.
"
" The first part of the tag ("XXX") is expected in the first column.
"
" A few examples:
" XXX title 01..12
" XXX full 20..31
" XXX full 90
"
" Please see "usr_toc.skeleton.txt" for a more complete example.
"
" Bugs: A few bugs remains about unwanted blank lines insertion when some
" files of the specified range are not found.
"
" History:
" 2005/01/17:	Creation.
" 2005/01/20:	Added options settings to speedup conversion.
" 2005/01/24:	Fixed bug in while loop in GenerateEntry ('0' insertion case).

let s:filename = "generated_usr_toc.txt"
let s:skelfile = "usr_toc.template.txt"
let s:rega = @a

function! s:Trim(string)
  return substitute(a:string, '^\s*\(.\{-}\)\s*$', '\1', '')
endfunction

function! ExtractSectionTitle(type)
  5
  yank a
  let ret = expand("%:t")
  let @a = s:Trim(@a)
  let @a = "|" . ret . "|  " . @a
  if a:type =~ "full"
    %g/^\*\d\d\.\d\d\=\*/yank A
    let @a = substitute(@a, '\*\(\d\d\.\d\d\=\)\*', '\t\t|\1|', 'g') . "\n"
    let @a = substitute(@a, '\s\+\*[^\*]\+\*', '', 'g')
  endif
  return @a
endfunction

function! ExtractInfo(file, type)
  if filereadable(a:file)
    execute "split " . a:file
    let ret = ExtractSectionTitle(a:type)
    bwipeout
    return ret
  else
    return ""
  endif
endfunction

function! GenerateEntry()
  " Get file(s) number.
  let metaline = getline(".")
  let istart = substitute(metaline, 
	\'XXX\s*\k*\s*\(\d\d\)\(\.\.\d\d\)\=', '\1', '')
  let istop = substitute(metaline,
	\'XXX\s*\k*\s*\d\d\.\.\(\d\d\)', '\1', '')
  " Get insertion type.
  if stridx(metaline, 'title') != -1
    let type = "title"
  else
    let type = "full"
  endif
  let @b = ""
  if istop =~ 'XXX'
    " Only one file in entry
    let @b = ExtractInfo("usr_" . istart . ".txt", type)
    delete
  else
    " Several files in entry
    while istart <= istop
      let @B = ExtractInfo("usr_" . (istart !~ '\d\d' ? "0" : "") .
	    \istart . ".txt", type)
      let istart = istart + 1
    endwhile
    delete
  endif
  put! b 
  if type =~ "full"
    " Too many lines at the end of the inserted text.
    /=====
    nohls
    normal kdd
  endif
endfunction


execute "split " . s:skelfile
execute "file " . s:filename

set lazyredraw
set noswapfile
set undolevels=-1
set eventignore=all
set nohidden
0
while search('^XXX', "W") != 0
  " I could have use :global, but another one is used
  " inside the loop, and it is not possible to nest
  " :global commands.
  call GenerateEntry()
endwhile

write
bwipeout

let @a = s:rega
unlet s:rega s:skelfile s:filename

" vim:sw=2:
